package function.builtin.special

import function.ArgumentValidator
import function.FunctionNames
import function.LispSpecialFunction
import function.UserDefinedFunction
import function.builtin.cons.LIST.makeList
import sexpression.Cons
import sexpression.LambdaExpression
import sexpression.SExpression
import sexpression.Symbol

@FunctionNames("LAMBDA", "Λ")
class LAMBDA(name: String) : LispSpecialFunction() {


    private val argumentValidator: ArgumentValidator = ArgumentValidator(name)
    private val lambdaListValidator: ArgumentValidator = ArgumentValidator("$name|lambda-list|")

    init {
        this.argumentValidator.setFirstArgumentExpectedType(Cons::class.java)
        this.argumentValidator.setMinimumNumberOfArguments(1)
        this.lambdaListValidator.setEveryArgumentExpectedType(Symbol::class.java)
    }

    override fun call(argumentList: Cons): LambdaExpression {
        argumentValidator.validate(argumentList)

        val first = argumentList.first
        val lambdaList = first as Cons
        val body = argumentList.rest as Cons

        lambdaListValidator.validate(lambdaList)

        val function = UserDefinedFunction(":LAMBDA", lambdaList, body)

        return LambdaExpression(makeOriginalLambdaExpression(argumentList), function)
    }

    private fun makeOriginalLambdaExpression(argumentList: Cons): Cons {
        return Cons(Symbol("LAMBDA"), argumentList)
    }

    companion object {

        fun isLambdaExpression(sexpr: SExpression): Boolean {
            if (sexpr.isCons) {
                val first = (sexpr as Cons).first.toString()

                return "LAMBDA" == first || "Λ" == first
            }

            return false
        }

        fun createFunction(lambdaExpression: Cons): UserDefinedFunction {
            val rest = lambdaExpression.rest

            val lambdaValidator = ArgumentValidator("LAMBDA|create|")
            lambdaValidator.setEveryArgumentExpectedType(Cons::class.java)
            lambdaValidator.validate(makeList(rest))

            val lambda = LAMBDA("LAMBDA").call(rest as Cons)

            return lambda.function
        }
    }
}
