package table

import org.assertj.core.api.Assertions.assertThat
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS
import sexpression.Nil.NIL
import sexpression.Symbol.T

@TestInstance(PER_CLASS)
class SymbolTableTest {

    private lateinit var symbolTable: SymbolTable

    @BeforeEach
    fun initialize() {
        symbolTable = SymbolTable()
    }

    @Test
    fun `lookup a symbol that does not exist`() {
        assertThat(symbolTable.contains("symbol")).isFalse()
    }

    @Test
    fun `lookup a symbol that exists`() {
        symbolTable.put("symbol", T)

        assertThat(symbolTable.contains("symbol")).isTrue()
    }

    @Test
    fun `get the value of a symbol`() {
        symbolTable.put("symbol", T)

        assertThat(symbolTable.get("symbol")).isEqualTo(T)
    }

    @Test
    fun `redefine the value of a symbol`() {
        symbolTable.put("symbol", T)
        symbolTable.put("symbol", NIL)

        assertThat(symbolTable.get("symbol")).isEqualTo(NIL)
    }

    @Test
    fun `check the value of a parent table`() {
        val childTable = SymbolTable(symbolTable)

        assertThat(childTable.parent).isEqualTo(symbolTable)
    }

    @Test
    fun `lookup a symbol in a parent table`() {
        symbolTable.put("symbol", T)
        val childTable = SymbolTable(symbolTable)
        val parentTable = childTable.parent

        assertThat(parentTable.get("symbol")).isEqualTo(T)
    }
}
